/**
 * @file
 * JavaScript behaviors for the donation widget.
 */

(function ($, Drupal, drupalSettings, once) {
  'use strict';

  Drupal.behaviors.commerceRecurringDonationWidget = {
    attach: function (context, settings) {
      
      // Keyboard navigation helper functions
      function setupKeyboardNavigation() {
        // Setup keyboard navigation for donation type selector (tabs)
        setupTabNavigation();
        
        // Setup keyboard navigation for donation amount options (radio group)
        setupRadioGroupNavigation();
        
        // Setup focus handlers for donation options
        setupDonationOptionFocusHandlers();
      }
      
      function setupTabNavigation() {
        var $tabs = $('.donation-type-option', context);
        
        $tabs.on('keydown', function(e) {
          var $current = $(this);
          var currentIndex = $tabs.index($current);
          var $target;
          
          switch(e.key) {
            case 'ArrowLeft':
            case 'ArrowUp':
              e.preventDefault();
              $target = currentIndex > 0 ? $tabs.eq(currentIndex - 1) : $tabs.last();
              break;
            case 'ArrowRight':
            case 'ArrowDown':
              e.preventDefault();
              $target = currentIndex < $tabs.length - 1 ? $tabs.eq(currentIndex + 1) : $tabs.first();
              break;
            case 'Home':
              e.preventDefault();
              $target = $tabs.first();
              break;
            case 'End':
              e.preventDefault();
              $target = $tabs.last();
              break;
            case 'Enter':
            case ' ':
              e.preventDefault();
              $current.trigger('click');
              return;
          }
          
          if ($target && $target.length) {
            // Update tabindex
            $tabs.attr('tabindex', '-1').attr('aria-selected', 'false');
            $target.attr('tabindex', '0').attr('aria-selected', 'true').focus();
          }
        });
      }
      
      function setupRadioGroupNavigation() {
        var $radioGroup = $('.donation-amounts', context);
        
        // Remove any existing handlers to prevent duplicates
        $radioGroup.off('keydown.donation-navigation');
        
        // Attach keydown handler directly to the donation amount options using event delegation
        $radioGroup.on('keydown.donation-navigation', '.donation-amount-option', function(e) {
          var $current = $(this);
          var $options = $('.donation-amount-option:visible', context);
          
          if ($options.length === 0) {
            return;
          }
          
          var currentIndex = $options.index($current);
          var $target;
          
          switch(e.key) {
            case 'ArrowLeft':
            case 'ArrowUp':
              e.preventDefault();
              $target = currentIndex > 0 ? $options.eq(currentIndex - 1) : $options.last();
              break;
            case 'ArrowRight':
            case 'ArrowDown':
              e.preventDefault();
              $target = currentIndex < $options.length - 1 ? $options.eq(currentIndex + 1) : $options.first();
              break;
            case 'Home':
              e.preventDefault();
              $target = $options.first();
              break;
            case 'End':
              e.preventDefault();
              $target = $options.last();
              break;
            case 'Enter':
            case ' ':
              e.preventDefault();
              $current.trigger('click');
              return;
          }
          
          if ($target && $target.length) {
            $target.focus();
          }
        });
      }
      
      function updateAriaStates($selectedOption) {
        var $options = $('.donation-amount-option:visible', context);
        
        // Update aria-checked states
        $options.attr('aria-checked', 'false').removeClass('active');
        $selectedOption.attr('aria-checked', 'true').addClass('active');
        
        // Update tabindex for proper tab order
        $options.attr('tabindex', '-1');
        $selectedOption.attr('tabindex', '0');
      }
      
      function setupDonationOptionFocusHandlers() {
        // Handle focus events on donation options to manage tabindex
        var $radioGroup = $('.donation-amounts', context);
        
        $radioGroup.off('focus.donation-options').on('focus.donation-options', '.donation-amount-option', function() {
          var $this = $(this);
          var $options = $('.donation-amount-option:visible', context);
          
          // Update tabindex so only the focused option is tabbable
          $options.attr('tabindex', '-1');
          $this.attr('tabindex', '0');
        });
      }
      
      function updateCustomAmountVisibility(show) {
        var $wrapper = $('.donation-custom-amount-input-wrapper', context);
        var $input = $('.custom-amount-input', context);
        
        if (show) {
          $wrapper.show().attr('aria-hidden', 'false');
          $input.focus();
        } else {
          $wrapper.hide().attr('aria-hidden', 'true');
        }
      }
      
      function updateDedicationFormVisibility(show) {
        var $container = $('.dedication-form-container', context);
        var $firstInput = $container.find('input[type="radio"]:first');
        
        if (show) {
          $container.show().attr('aria-hidden', 'false');
          // Focus the first radio button when showing
          setTimeout(function() {
            $firstInput.focus();
          }, 100);
        } else {
          $container.hide().attr('aria-hidden', 'true');
        }
      }
      
      function announceToScreenReader(message) {
        var $announcement = $('<div>', {
          'aria-live': 'polite',
          'aria-atomic': 'true',
          'class': 'sr-only'
        }).text(message);
        
        $('body').append($announcement);
        
        setTimeout(function() {
          $announcement.remove();
        }, 1000);
      }

      // Toggle custom amount input.
      function toggleCustomInput($optionElement, show) {
        updateCustomAmountVisibility(show);
      }

      // Update donation amounts based on type.
      function updateDonationAmounts(donationType) {
        if (drupalSettings.commerceRecurringDonationWidget &&
            drupalSettings.commerceRecurringDonationWidget.donationOptionsByType &&
            drupalSettings.commerceRecurringDonationWidget.donationOptionsByType[donationType]) {

          var optionsForType = drupalSettings.commerceRecurringDonationWidget.donationOptionsByType[donationType];
          var $donationAmountsContainer = $('.donation-amounts', context);
          $donationAmountsContainer.empty();

          var customAmountVariationId = null;

          $.each(optionsForType, function(index, option) {
            var $option = $('<div class="donation-amount-option"></div>')
              .attr('data-amount', option.amount)
              .attr('data-variation-id', option.variation_id)
              .attr('data-donation-type', option.type)
              .attr('data-is-custom-option', option.is_custom_amount ? 'true' : 'false')
              .attr('data-option-title', option.title.toLowerCase())
              .attr('role', 'radio')
              .attr('tabindex', index === 0 ? '0' : '-1')
              .attr('aria-checked', 'false')
              .text(option.title);

            if (option.is_custom_amount) {
              customAmountVariationId = option.variation_id;
              $option.attr('aria-describedby', 'custom-amount-description');
            }

            $donationAmountsContainer.append($option);

            $option.on('click', function() {
              var $this = $(this);
              var isCustomOption = $this.data('is-custom-option') === true || $this.data('is-custom-option') === 'true';
              
              updateAriaStates($this);
              
              $('.donation-amount-option', context).not($this).each(function() {
                toggleCustomInput($(this), false);
              });

              if (isCustomOption) {
                toggleCustomInput($this, true);
                announceToScreenReader(Drupal.t('Custom amount option selected. Please enter your amount.'));
              } else {
                toggleCustomInput($this, false);
                $('.donation-custom-amount-input-wrapper', context).find('.custom-amount-input').val('');
                announceToScreenReader(Drupal.t('Amount @amount selected', {'@amount': option.title}));
              }
            });
          });

          if (customAmountVariationId) {
            $('.custom-amount-input', context).attr('data-variation-id-for-custom', customAmountVariationId);
          }

          if (optionsForType.length === 1) {
            $('.donation-amount-option', context).first().trigger('click');
          }
        } else {
          // Fallback behavior.
          $('.donation-amount-option', context).each(function() {
            var $option = $(this);
            if ($option.data('donation-type') === donationType) {
              $option.removeClass('hidden').attr('aria-hidden', 'false');
            } else {
              $option.addClass('hidden').attr('aria-hidden', 'true');
              toggleCustomInput($option, false);
            }
          });

          var $visibleOptions = $('.donation-amount-option', context).filter(function() {
            return $(this).data('donation-type') === donationType && !$(this).hasClass('hidden');
          });

          // Update tabindex for visible options
          $visibleOptions.each(function(index) {
            $(this).attr('tabindex', index === 0 ? '0' : '-1');
          });

          if ($visibleOptions.length === 1) {
            var $singleOption = $visibleOptions.first();
            if (!$singleOption.hasClass('active')) {
              $singleOption.trigger('click');
            }
          }
        }
      }

      // Initialize based on active donation type from settings
      var initialDonationType = $('.donation-type-option.active', context).data('donation-type');
      if (initialDonationType) {
        updateDonationAmounts(initialDonationType);
      }
      
      // Ensure initially no custom inputs are shown unless an option is active and custom
      $('.donation-custom-amount-input-wrapper', context).hide().attr('aria-hidden', 'true');
      $('.donation-amount-option.active', context).each(function() {
        var $this = $(this);
        var isCustom = $this.data('is-custom-option') === true || $this.data('is-custom-option') === 'true';
        if (isCustom) {
          toggleCustomInput($this, true);
        }
      });

      // Setup keyboard navigation
      setupKeyboardNavigation();

      // Switch between donation types (one-time, monthly, yearly)
      once('donation-type-handler', $('.donation-type-option', context)).forEach(function (element) {
        $(element).on('click', function () {
          var $this = $(this);
          var donationType = $this.data('donation-type');

          // Update active state for type selectors
          $('.donation-type-option', context).removeClass('active').attr('aria-selected', 'false').attr('tabindex', '-1');
          $this.addClass('active').attr('aria-selected', 'true').attr('tabindex', '0');

          // Deselect any selected amount option and clear/hide all custom inputs BEFORE updating amounts
          $('.donation-amount-option', context).removeClass('active').attr('aria-checked', 'false');
          $('.donation-custom-amount-input-wrapper', context).hide().attr('aria-hidden', 'true').find('.custom-amount-input').val('');

          // Now update and potentially auto-select amounts for the new type
          updateDonationAmounts(donationType);
          
          // Announce the change
          announceToScreenReader(Drupal.t('Switched to @type donations', {'@type': $this.text()}));
        });
      });

      // Handle predefined donation amount selection
      once('donation-amount-handler', $('.donation-amount-option', context)).forEach(function (element) {
        $(element).on('click', function () {
          var $this = $(this);
          var isCustomOption = $this.data('is-custom-option') === true || $this.data('is-custom-option') === 'true';

          updateAriaStates($this);

          // Deactivate other options and their custom inputs
          $('.donation-amount-option', context).not($this).each(function() {
            toggleCustomInput($(this), false);
          });

          if (isCustomOption) {
            toggleCustomInput($this, true);
            // Do not clear this specific input, user might be re-clicking or focusing
          } else {
            // It's a non-custom option, ensure its specific custom input (if it were one) is hidden
            // And clear all other custom inputs
            toggleCustomInput($this, false); // Hide its own (if it was custom)
            $('.donation-custom-amount-input-wrapper', context).find('.custom-amount-input').val('');
          }
        });
      });

      // Handle custom amount input field interaction
      // When user types in a custom input, its associated "Custom" option should remain active.
      // Clearing other custom inputs is handled by the option click.
      once('custom-amount-input-interaction', $('.custom-amount-input', context)).forEach(function(inputElement) {
        $(inputElement).on('input focus', function() {
          // Find the custom option
          var $customOption = $('.donation-amount-option', context).filter(function() {
            return ($(this).data('is-custom-option') === true || $(this).data('is-custom-option') === 'true');
          });

          if ($customOption.length) {
            if (!$customOption.hasClass('active')) {
              // Activate the custom option if user interacts with the input
              updateAriaStates($customOption);
              $('.donation-amount-option', context).not($customOption).removeClass('active');
            }
          }
        });
      });

      // Toggle dedication form
      once('dedication-toggle', $('#enable-dedication', context)).forEach(function (element) {
        $(element).on('change', function () {
          var isChecked = this.checked;
          updateDedicationFormVisibility(isChecked);
          
          if (isChecked) {
            announceToScreenReader(Drupal.t('Dedication form opened'));
          } else {
            announceToScreenReader(Drupal.t('Dedication form closed'));
          }
        });
      });

      // Switch between dedication types (honor/memory)
      once('dedication-type-handler', $('.dedication-type-option', context)).forEach(function (element) {
        var $option = $(element);
        
        // Set initial active state based on checked radio
        if ($option.find('input[type="radio"]').is(':checked')) {
          $option.addClass('active');
        }
        
        $option.on('click', function(e) {
          // Only proceed if the click wasn't directly on the radio input
          // (to avoid duplicate events since clicking the label also triggers the radio)
          if (e.target.tagName.toLowerCase() !== 'input') {
            var $radio = $option.find('input[type="radio"]');
            $radio.prop('checked', true).trigger('change');
          }
        });
        
        // Update active state when radio changes
        $option.find('input[type="radio"]').on('change', function() {
          if ($(this).is(':checked')) {
            $('.dedication-type-option', context).removeClass('active');
            $option.addClass('active');
            
            var dedicationType = $(this).val();
            var message = dedicationType === 'honor' ? 
              Drupal.t('In honor of selected') : 
              Drupal.t('In memory of selected');
            announceToScreenReader(message);
          }
        });
      });

      // Handle the next button click
      once('donation-next-handler', $('.donation-next-button', context)).forEach(function (element) {
        $(element).on('click', function () {
          var $activeDonationTypeOption = $('.donation-type-option.active', context);
          if (!$activeDonationTypeOption.length) {
            var message = Drupal.t('Please select a donation frequency (e.g., Monthly).');
            alert(message);
            announceToScreenReader(message);
            $('.donation-type-option', context).first().focus();
            return;
          }
          var donationType = $activeDonationTypeOption.data('donation-type');

          var $selectedOption = $('.donation-amount-option.active', context);
          if (!$selectedOption.length) {
            var message = Drupal.t('Please select a donation option.');
            alert(message);
            announceToScreenReader(message);
            $('.donation-amount-option:visible', context).first().focus();
            return;
          }

          var variationId = $selectedOption.data('variation-id');
          var amount;
          var isCustomSelected = ($selectedOption.data('is-custom-option') === true || $selectedOption.data('is-custom-option') === 'true');

          if (isCustomSelected) {
            // Find the custom amount input anywhere in the widget, not just as a sibling
            var $customInput = $('.donation-custom-amount-input-wrapper', context).find('.custom-amount-input');
            var customAmountValue = $customInput.val() ? $customInput.val().trim() : '';

            if (customAmountValue === '') {
              var message = Drupal.t('Please enter your custom donation amount.');
              alert(message);
              announceToScreenReader(message);
              $customInput.focus();
              return;
            }
            var parsedCustomAmount = parseFloat(customAmountValue);
            if (isNaN(parsedCustomAmount) || !isFinite(parsedCustomAmount) || parsedCustomAmount <= 0) {
              var message = Drupal.t('Please enter a valid number greater than zero for your custom amount.');
              alert(message);
              announceToScreenReader(message);
              $customInput.focus();
              return;
            }
            amount = parsedCustomAmount;
            // variationId is already set from $selectedOption
          } else {
            // Regular predefined option
            var amountString = $selectedOption.data('amount').toString();
            var parsedAmount = parseFloat(amountString); // Should be clean number from preprocess
            if (!isNaN(parsedAmount) && isFinite(parsedAmount) && parsedAmount > 0) {
              amount = parsedAmount;
            } else {
              // This case should ideally not happen if preprocess prepares 'amount' correctly for non-custom options
              var message = Drupal.t('The selected option does not have a valid amount: ') + $selectedOption.text();
              alert(message);
              announceToScreenReader(message);
              return;
            }
          }

          if (!variationId || typeof amount === 'undefined' || amount === null) {
            var message = Drupal.t('Could not determine the variation ID or amount. Please try again.');
            alert(message);
            announceToScreenReader(message);
            return;
          }

          var designation = '';
          if ($('#donation-designation', context).length) {
            designation = $('#donation-designation', context).val();
          }

          var dedicationData = null;
          if ($('#enable-dedication', context).is(':checked')) {
            // Get the value directly from the checked radio button instead of the active class
            var dedicationType = $('input[name="dedication-type"]:checked', context).val();
            var dedicationName = $('#dedication-name', context).val();
            
            if (!dedicationName || dedicationName.trim() === '') {
              var message = Drupal.t('Please enter a name for the dedication.');
              alert(message);
              announceToScreenReader(message);
              $('#dedication-name', context).focus();
              return;
            }
            
            dedicationData = {
              type: dedicationType,
              dedicationName: dedicationName,
              message: $('#dedication-message', context).val()
            };
          }

          var comment = '';
          if ($('#donation-comment', context).length) {
            comment = $('#donation-comment', context).val();
          }

          var data = {
            variation_id: variationId,
            amount: amount, // This will be the custom numeric amount or parsed numeric amount
            donation_type: donationType,
            designation: designation,
            dedication: dedicationData,
            comment: comment
          };

          // Announce that submission is in progress
          announceToScreenReader(Drupal.t('Submitting your donation...'));

          $.ajax({
            url: drupalSettings.commerceRecurringDonationWidget.addToCartUrl,
            type: 'POST',
            data: JSON.stringify(data),
            contentType: 'application/json',
            dataType: 'json',
            success: function (response) {
              if (response.redirect) {
                announceToScreenReader(Drupal.t('Donation submitted successfully. Redirecting to cart.'));
                window.location.href = response.redirect;
              } else if (response.error) {
                var message = Drupal.t('Error: ') + response.error;
                alert(message);
                announceToScreenReader(message);
              } else {
                // Consider a more specific success message or action if not redirecting.
                var message = Drupal.t('Action completed. Please check your cart or messages.');
                alert(message);
                announceToScreenReader(message);
              }
            },
            error: function (jqXHR, textStatus, errorThrown) {
              var errorMessage = Drupal.t('An error occurred. Please try again.');
              if (jqXHR.responseJSON && jqXHR.responseJSON.error) {
                  errorMessage = jqXHR.responseJSON.error;
              }
              alert(errorMessage);
              announceToScreenReader(errorMessage);
            }
          });
        });
      });
    }
  };

  /**
   * Behaviors for donation field editing (dedication, designation, comment).
   */
  Drupal.behaviors.donationFieldEdit = {
    attach: function (context, settings) {
      // Watch for dedication form submissions
      $(once('dedication-form-submit', 'form.dedication-edit-form', context)).on('submit', function (event) {
        event.preventDefault();
        
        var $form = $(this);
        var orderItemId = $form.find('input[name="order_item_id"]').val();
        var field = $form.find('input[name="field"]').val();
        var value = $form.find('[name="value"]').val();
        
        // Add dedication type if it's in the form
        var data = {
          order_item_id: orderItemId,
          field: field,
          value: value
        };
        
        if ($form.find('input[name="dedication_type"]:checked').length) {
          data.dedication_type = $form.find('input[name="dedication_type"]:checked').val();
        }
        
        // Submit the data via AJAX
        $.ajax({
          url: Drupal.url('commerce-recurring-donation-widget/dedication/update'),
          type: 'POST',
          data: JSON.stringify(data),
          contentType: 'application/json',
          dataType: 'json',
          success: function(response) {
            if (response.success) {
              // Close the modal dialog
              if ($form.closest('.ui-dialog-content').length) {
                $form.closest('.ui-dialog-content').dialog('close');
              }
              
              // We don't need to refresh the entire cart view since our PHP callback 
              // now updates only the specific dedication elements
            } else if (response.error) {
              Drupal.behaviors.donationFieldEdit.showMessage(response.error, 'error');
            }
          },
          error: function(xhr) {
            var errorMessage = Drupal.t('An error occurred while updating the dedication.');
            if (xhr.responseJSON && xhr.responseJSON.error) {
              errorMessage = xhr.responseJSON.error;
            }
            Drupal.behaviors.donationFieldEdit.showMessage(errorMessage, 'error');
          }
        });
      });
      
      // Watch for designation form submissions
      $(once('designation-form-submit', 'form.designation-edit-form', context)).on('submit', function (event) {
        event.preventDefault();
        
        var $form = $(this);
        var orderItemId = $form.find('input[name="order_item_id"]').val();
        var designation = $form.find('select[name="designation"]').val();
        
        // Submit the data via AJAX
        $.ajax({
          url: Drupal.url('commerce-recurring-donation-widget/designation/update'),
          type: 'POST',
          data: JSON.stringify({
            order_item_id: orderItemId,
            designation: designation
          }),
          contentType: 'application/json',
          dataType: 'json',
          success: function(response) {
            if (response.success) {
              // Close the modal dialog
              if ($form.closest('.ui-dialog-content').length) {
                $form.closest('.ui-dialog-content').dialog('close');
              }
              
              // The designation will be updated via the AJAX response from the form submit handler
            } else if (response.error) {
              Drupal.behaviors.donationFieldEdit.showMessage(response.error, 'error');
            }
          },
          error: function(xhr) {
            var errorMessage = Drupal.t('An error occurred while updating the designation.');
            if (xhr.responseJSON && xhr.responseJSON.error) {
              errorMessage = xhr.responseJSON.error;
            }
            Drupal.behaviors.donationFieldEdit.showMessage(errorMessage, 'error');
          }
        });
      });

      // Watch for comment form submissions
      $(once('comment-form-submit', 'form.comment-edit-form', context)).on('submit', function (event) {
        event.preventDefault();
        
        var $form = $(this);
        var orderItemId = $form.find('input[name="order_item_id"]').val();
        var comment = $form.find('[name="comment"]').val();
        
        // Submit the data via AJAX
        $.ajax({
          url: Drupal.url('commerce-recurring-donation-widget/comment/update'),
          type: 'POST',
          data: JSON.stringify({
            order_item_id: orderItemId,
            comment: comment
          }),
          contentType: 'application/json',
          dataType: 'json',
          success: function(response) {
            if (response.success) {
              // Close the modal dialog
              if ($form.closest('.ui-dialog-content').length) {
                $form.closest('.ui-dialog-content').dialog('close');
              }
              
              // The comment will be updated via the AJAX response from the form submit handler
            } else if (response.error) {
              Drupal.behaviors.donationFieldEdit.showMessage(response.error, 'error');
            }
          },
          error: function(xhr) {
            var errorMessage = Drupal.t('An error occurred while updating the comment.');
            if (xhr.responseJSON && xhr.responseJSON.error) {
              errorMessage = xhr.responseJSON.error;
            }
            Drupal.behaviors.donationFieldEdit.showMessage(errorMessage, 'error');
          }
        });
      });
    },
    
    // Helper function to show messages
    showMessage: function(message, type) {
      if (typeof Drupal.message !== 'undefined' && Drupal.message.announce) {
        Drupal.message.announce(message, type);
      } else {
        alert(message);
      }
    }
  };
})(jQuery, Drupal, drupalSettings, once);
