<?php

namespace Drupal\commerce_recurring_donation_widget\Controller;

use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;

/**
 * Controller for handling order item comment edits.
 */
class CommentEditController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new CommentEditController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Handles access checks for the edit endpoint.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account) {
    // Allow access for all users (authenticated and anonymous) who can access content.
    // This is needed for donation widgets to work for anonymous users.
    return AccessResult::allowedIfHasPermission($account, 'access content');
  }

  /**
   * Get modal form for editing comment field.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   * @param int $order_item_id
   *   The order item ID.
   *
   * @return array
   *   A render array for the modal form.
   */
  public function getModalForm(Request $request, $order_item_id) {
    // Load the order item.
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($order_item_id);
    if (!$order_item) {
      return new JsonResponse(['error' => 'Order item not found'], 404);
    }

    // Get current comment value.
    $current_value = '';
    if ($order_item->hasField('field_donation_comment')) {
      $current_value = $order_item->get('field_donation_comment')->value;
    }

    // Build the form.
    $form = $this->formBuilder()->getForm(
      'Drupal\commerce_recurring_donation_widget\Form\CommentEditForm',
      $order_item_id,
      $current_value
    );

    // For AJAX requests, return as a dialog.
    if ($request->isXmlHttpRequest()) {
      $response = new AjaxResponse();
      $response->addCommand(new OpenModalDialogCommand(
        $this->t('Edit Comment'),
        $form,
        [
          'width' => '500',
          'dialogClass' => 'comment-edit-dialog',
          'closeOnEscape' => TRUE,
          'modal' => TRUE,
          'autoResize' => TRUE,
        ]
      ));
      return $response;
    }

    // For non-AJAX requests, return the form in a standard page.
    return [
      '#type' => 'page',
      '#title' => $this->t('Edit Comment'),
      'content' => $form,
    ];
  }

  /**
   * Updates the comment field value.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response.
   */
  public function updateField(Request $request) {
    $data = json_decode($request->getContent(), TRUE);

    if (empty($data['order_item_id'])) {
      return new JsonResponse(['error' => $this->t('Missing required order item ID')], 400);
    }

    // Comment can be empty (data['comment'] can be empty)
    try {
      // Load the order item.
      /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
      $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($data['order_item_id']);

      if (!$order_item) {
        return new JsonResponse(['error' => $this->t('Order item not found')], 404);
      }

      // Check that the order is still in draft state.
      $order = $order_item->getOrder();
      if (!$order || $order->getState()->getId() !== 'draft') {
        return new JsonResponse(['error' => $this->t('Order cannot be modified')], 403);
      }

      // Update the comment field.
      if ($order_item->hasField('field_donation_comment')) {
        $order_item->set('field_donation_comment', $data['comment'] ?? '');
      }

      $order_item->save();

      // Prepare response with updated values for display.
      $response_data = [
        'success' => TRUE,
        'order_item_id' => $order_item->id(),
        'comment' => $data['comment'] ?? '',
      ];

      return new JsonResponse($response_data);
    }
    catch (\Exception $e) {
      $this->getLogger('commerce_recurring_donation_widget')
        ->error('Error updating comment field: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => $this->t('An unexpected error occurred')], 500);
    }
  }

}
