<?php

namespace Drupal\commerce_recurring_donation_widget\Controller;

use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Form\FormState;

/**
 * Controller for handling dedication field edits.
 */
class DedicationEditController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new DedicationEditController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Handles access checks for the edit endpoint.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account) {
    // Allow access for all users (authenticated and anonymous) who can access content.
    // This is needed for donation widgets to work for anonymous users.
    return AccessResult::allowedIfHasPermission($account, 'access content');
  }

  /**
   * Get modal form for editing dedication fields.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   * @param int $order_item_id
   *   The order item ID.
   * @param string $field
   *   The field name to edit.
   *
   * @return array
   *   A render array for the modal form.
   */
  public function getModalForm(Request $request, $order_item_id, $field) {
    $build = [];

    // Load the order item.
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($order_item_id);
    if (!$order_item) {
      return new JsonResponse(['error' => 'Order item not found'], 404);
    }

    // Get current values.
    $current_value = '';
    $dedication_type = '';

    if ($order_item->hasField($field)) {
      $current_value = $order_item->get($field)->value;
    }

    if ($order_item->hasField('field_dedication_type')) {
      $dedication_type = $order_item->get('field_dedication_type')->value;
    }

    // Build the form.
    $build['form'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'dedication-edit-form',
        'class' => ['dedication-edit-form'],
      ],
    ];

    $build['form']['order_item_id'] = [
      '#type' => 'hidden',
      '#value' => $order_item_id,
    ];

    $build['form']['field'] = [
      '#type' => 'hidden',
      '#value' => $field,
    ];

    $title = '';
    if ($field == 'field_dedication_name') {
      $title = $this->t('Edit dedication name');
      $build['form']['value'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Name'),
        '#default_value' => $current_value,
        '#required' => TRUE,
      ];
    }
    elseif ($field == 'field_dedication_message') {
      $title = $this->t('Edit dedication message');
      $build['form']['value'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Message'),
        '#default_value' => $current_value,
        '#rows' => 4,
      ];
    }

    // If editing the name, allow changing the dedication type too.
    if ($field == 'field_dedication_name') {
      $build['form']['dedication_type'] = [
        '#type' => 'radios',
        '#title' => $this->t('Dedication type'),
        '#options' => [
          'honor' => $this->t('In honor of'),
          'memory' => $this->t('In memory of'),
        ],
        '#default_value' => $dedication_type,
        '#required' => TRUE,
      ];
    }

    $build['form']['actions'] = [
      '#type' => 'actions',
    ];

    $build['form']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#attributes' => [
        'class' => ['dedication-edit-submit'],
      ],
    ];

    $build['form']['actions']['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#attributes' => [
        'class' => ['dedication-edit-cancel'],
        'data-dismiss' => 'modal',
      ],
    ];

    // Build the form with the appropriate theme wrapper for dialogs
    // Create a proper form state object.
    $form_state = new FormState();
    $form_state->set('order_item_id', $order_item_id);
    $form_state->set('field', $field);
    $form_state->set('current_value', $current_value);
    $form_state->set('dedication_type', $dedication_type);
    $form_state->set('title', $title);

    $form = $this->formBuilder()->buildForm('Drupal\commerce_recurring_donation_widget\Form\DedicationEditForm', $form_state);

    // For AJAX requests, return as a dialog.
    if ($request->isXmlHttpRequest()) {
      $response = new AjaxResponse();
      $response->addCommand(new OpenModalDialogCommand($title, $form, [
        'width' => '500',
        'dialogClass' => 'dedication-edit-dialog',
        'closeOnEscape' => TRUE,
        'modal' => TRUE,
        'autoResize' => TRUE,
      ]));
      return $response;
    }

    // For non-AJAX requests, return the form in a standard page.
    return [
      '#type' => 'page',
      '#title' => $title,
      'content' => $form,
    ];
  }

  /**
   * Updates the dedication field value.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response.
   */
  public function updateField(Request $request) {
    $data = json_decode($request->getContent(), TRUE);

    if (empty($data['order_item_id']) || empty($data['field'])) {
      return new JsonResponse(['error' => $this->t('Missing required data')], 400);
    }

    try {
      // Load the order item.
      /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
      $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($data['order_item_id']);

      if (!$order_item) {
        return new JsonResponse(['error' => $this->t('Order item not found')], 404);
      }

      // Check that the order is still in draft state.
      $order = $order_item->getOrder();
      if (!$order || $order->getState()->getId() !== 'draft') {
        return new JsonResponse(['error' => $this->t('Order cannot be modified')], 403);
      }

      // Update the field.
      if ($order_item->hasField($data['field'])) {
        $order_item->set($data['field'], $data['value']);
      }

      // Also update dedication type if provided and editing the name.
      if (!empty($data['dedication_type']) && $data['field'] == 'field_dedication_name' && $order_item->hasField('field_dedication_type')) {
        $order_item->set('field_dedication_type', $data['dedication_type']);
      }

      $order_item->save();

      // Prepare response with updated values for display.
      $response_data = [
        'success' => TRUE,
        'order_item_id' => $order_item->id(),
        'field' => $data['field'],
        'value' => $data['value'],
      ];

      if ($data['field'] == 'field_dedication_name') {
        $response_data['dedication_type'] = $order_item->get('field_dedication_type')->value;
        $response_data['dedication_type_label'] = ($response_data['dedication_type'] == 'honor')
          ? $this->t('In honor of')
          : $this->t('In memory of');
      }

      return new JsonResponse($response_data);
    }
    catch (\Exception $e) {
      $this->getLogger('commerce_recurring_donation_widget')
        ->error('Error updating dedication field: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => $this->t('An unexpected error occurred')], 500);
    }
  }

}
