<?php

namespace Drupal\commerce_recurring_donation_widget\Controller;

use Drupal\commerce_store\Entity\StoreInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_price\Price;
// Use Drupal\state_machine\WorkflowManagerInterface; // Removed as per previous diff if not used.
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for handling donation widget operations.
 */
class DonationWidgetController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  /**
   * The cart provider.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * Constructs a new DonationWidgetController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_cart\CartManagerInterface $cart_manager
   *   The cart manager.
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   The cart provider.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    CartManagerInterface $cart_manager,
    CartProviderInterface $cart_provider,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->cartManager = $cart_manager;
    $this->cartProvider = $cart_provider;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('commerce_cart.cart_manager'),
      $container->get('commerce_cart.cart_provider')
    );
  }

  /**
   * Adds the donation to the cart.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response.
   */
  public function addToCart(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    if (empty($data['variation_id']) || !isset($data['amount'])) {
      return new JsonResponse(['error' => $this->t('Invalid request data.')], 400);
    }

    try {
      $variation_storage = $this->entityTypeManager->getStorage('commerce_product_variation');
      /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $variation */
      $variation = $variation_storage->load($data['variation_id']);

      if (!$variation || !$variation->isPublished()) {
        return new JsonResponse(['error' => $this->t('Invalid or inactive product variation.')], 400);
      }

      // Get the current store.
      $current_store_service = \Drupal::service('commerce_store.current_store');
      try {
        $store = $current_store_service->getStore();
      } catch (\Exception $e) {
        $store_storage = $this->entityTypeManager->getStorage('commerce_store');
        $stores = $store_storage->loadMultiple();
        if (!empty($stores)) {
          $store = reset($stores);
        } else {
          return new JsonResponse(['error' => $this->t('Store not found.')], 500);
        }
      }

      // Get or create donation cart.
      $cart = $this->cartProvider->getCart('donation', $store);
      if (!$cart || $cart->isLocked() || $cart->getState()->getId() !== 'draft') {
        $cart = $this->cartProvider->createCart('donation', $store);
      }

      // Create line item.
      $order_item_type_storage = $this->entityTypeManager->getStorage('commerce_order_item_type');
      $donation_type = $order_item_type_storage->load('donation');
      $order_item_type_id = $donation_type ? 'donation' : $variation->getOrderItemTypeId();

      /** @var \Drupal\commerce_order\Entity\OrderItemInterface $line_item */
      $line_item = $this->entityTypeManager->getStorage('commerce_order_item')->create([
        'type' => $order_item_type_id,
        'purchased_entity' => $variation->id(),
        'quantity' => 1,
      ]);

      // Set custom unit price.
      if (isset($data['amount']) && is_numeric($data['amount']) && (float) $data['amount'] > 0) {
        $amount_string = (string) $data['amount'];
        $currency_code = $variation->getPrice()->getCurrencyCode();

        if (empty($currency_code)) {
          if ($store instanceof StoreInterface && method_exists($store, 'getDefaultCurrencyCode')) {
            $currency_code = $store->getDefaultCurrencyCode();
          }
          if (empty($currency_code)) {
            $currency_code = 'EUR';
          }
        }
        $custom_price = new Price($amount_string, $currency_code);
        $line_item->setUnitPrice($custom_price, TRUE);
      }

      // Set billing schedule for recurring donations.
      if (!empty($data['donation_type']) && $data['donation_type'] !== 'one_time') {
        $billing_schedule_storage = $this->entityTypeManager->getStorage('commerce_billing_schedule');
        $schedules = $billing_schedule_storage->loadByProperties(['plugin' => $data['donation_type']]);
        if (!empty($schedules)) {
          $billing_schedule = reset($schedules);
          if ($billing_schedule && $line_item->hasField('billing_schedule')) {
            $line_item->set('billing_schedule', $billing_schedule);
          }
        }
      }

      // Set dedication fields.
      if (!empty($data['dedication'])) {
        $dedication = $data['dedication'];
        if ($line_item->hasField('field_dedication_type') && !empty($dedication['type'])) {
          $line_item->set('field_dedication_type', $dedication['type']);
        }
        if ($line_item->hasField('field_dedication_name') && !empty($dedication['dedicationName'])) {
          $line_item->set('field_dedication_name', $dedication['dedicationName']);
        }
        if ($line_item->hasField('field_dedication_message') && !empty($dedication['message'])) {
          $line_item->set('field_dedication_message', $dedication['message']);
        }
      }

      // Set designation and comment.
      if (!empty($data['designation']) && $line_item->hasField('field_designation')) {
        $line_item->set('field_designation', $data['designation']);
      }

      if (!empty($data['comment']) && $line_item->hasField('field_donation_comment')) {
        $line_item->set('field_donation_comment', $data['comment']);
      }

      // Set descriptive title and add to cart.
      $this->setDonationTitle($line_item, $data);
      $this->cartManager->addOrderItem($cart, $line_item, TRUE);
      $cart->save();

      return new JsonResponse([
        'cart_id' => $cart->id(),
        'redirect' => '/cart',
      ]);
    }
    catch (\Exception $e) {
      $this->getLogger('commerce_recurring_donation_widget')->error('Error in addToCart: @message (file: @file, line: @line)', [
        '@message' => $e->getMessage(),
        '@file' => $e->getFile(),
        '@line' => $e->getLine(),
        'exception' => $e,
      ]);
      return new JsonResponse(['error' => $this->t('An unexpected error occurred. Please try again.')], 500);
    }
  }

  /**
   * Sets a descriptive title for the donation order item and product variation.
   *
   * @param \Drupal\commerce_order\Entity\OrderItemInterface $line_item
   *   The order item.
   * @param array $data
   *   The donation data from the request.
   */
  protected function setDonationTitle($line_item, array $data) {
    $amount = $data['amount'] ?? 0;
    $donation_type = $data['donation_type'] ?? 'one_time';
    
    // Get currency code.
    $currency_code = 'USD';
    $unit_price = $line_item->getUnitPrice();
    if ($unit_price) {
      $currency_code = $unit_price->getCurrencyCode();
    } else {
      $purchased_entity = $line_item->getPurchasedEntity();
      if ($purchased_entity && $purchased_entity->getPrice()) {
        $currency_code = $purchased_entity->getPrice()->getCurrencyCode();
      }
    }

    // Format amount.
    $currency_formatter = \Drupal::service('commerce_price.currency_formatter');
    $formatted_amount = $currency_formatter->format($amount, $currency_code);

    // Get type labels.
    $config = \Drupal::config('commerce_recurring_donation_widget.settings');
    $donation_type_labels = $config->get('donation_type_labels') ?: [];
    $default_labels = [
      'one_time' => $this->t('One-time'),
      'monthly' => $this->t('Monthly'),
      'yearly' => $this->t('Yearly'),
    ];
    $type_label = $donation_type_labels[$donation_type] ?? $default_labels[$donation_type] ?? $this->t('One-time');
    
    // Create title.
    $title = $this->t('@type Donation - @amount', [
      '@type' => $type_label,
      '@amount' => $formatted_amount,
    ]);

    $line_item->setTitle($title);
    
    // Update variation and product titles.
    $purchased_entity = $line_item->getPurchasedEntity();
    if ($purchased_entity) {
      if ($purchased_entity->hasField('title')) {
        $purchased_entity->set('title', $title);
        $purchased_entity->save();
      }
      
      if ($purchased_entity->hasField('product_id')) {
        $product = $purchased_entity->get('product_id')->entity;
        if ($product && $product->hasField('title')) {
          $product->set('title', $title);
          $product->save();
        }
      }
    }
  }



}
