<?php

namespace Drupal\commerce_recurring_donation_widget\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;

/**
 * Form for editing order item comments.
 */
class CommentEditForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The order item ID.
   *
   * @var int
   */
  protected $orderItemId;

  /**
   * The current comment value.
   *
   * @var string
   */
  protected $currentValue;

  /**
   * Constructs a new CommentEditForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'comment_edit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $order_item_id = NULL, $current_value = NULL) {
    $this->orderItemId = $order_item_id;
    $this->currentValue = $current_value;

    // Add the form attributes for AJAX.
    $form['#attributes']['class'][] = 'comment-edit-form';

    // Store order item ID for submit handler.
    $form['order_item_id'] = [
      '#type' => 'hidden',
      '#value' => $this->orderItemId,
    ];

    // Comment text area.
    $form['comment'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Comment'),
      '#default_value' => $this->currentValue,
      '#required' => FALSE,
      '#rows' => 4,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
      '#attributes' => [
        'class' => ['button--primary'],
      ],
      '#ajax' => [
        'callback' => '::ajaxSubmitCallback',
        'event' => 'click',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Updating...'),
        ],
      ],
    ];

    $form['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#button_type' => 'secondary',
      '#attributes' => [
        'class' => ['dialog-cancel'],
      ],
    ];

    // Add a wrapper for error messages.
    $form['status_messages'] = [
      '#type' => 'status_messages',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $order_item_id = $form_state->getValue('order_item_id');
    $comment = $form_state->getValue('comment');

    try {
      // Load the order item.
      /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
      $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($order_item_id);

      if (!$order_item) {
        $this->messenger()->addError($this->t('Order item not found'));
        return;
      }

      // Check that the order is still in draft state.
      $order = $order_item->getOrder();
      if (!$order || $order->getState()->getId() !== 'draft') {
        $this->messenger()->addError($this->t('Order cannot be modified'));
        return;
      }

      // Update the comment field.
      if ($order_item->hasField('field_donation_comment')) {
        $order_item->set('field_donation_comment', $comment);
        $order_item->save();
      }
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('An error occurred: @error', ['@error' => $e->getMessage()]));
      $this->getLogger('commerce_recurring_donation_widget')->error('Error updating comment: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * AJAX callback for the form submission.
   */
  public function ajaxSubmitCallback(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    // If there are any errors, re-display the form.
    if ($form_state->hasAnyErrors()) {
      $response->addCommand(new ReplaceCommand('.comment-edit-form', $form));
      return $response;
    }

    $order_item_id = $form_state->getValue('order_item_id');
    $comment_value = trim($form_state->getValue('comment'));

    // Close the dialog.
    $response->addCommand(new CloseModalDialogCommand());

    // Prepare the HTML for the content that will go INSIDE .donation-comment
    $selector = '[data-order-item-id="' . $order_item_id . '"].donation-comment';
    $new_content_html = '';

    if (empty($comment_value)) {
      // HTML for "Add Comment" link.
      $add_link_url = Url::fromRoute('commerce_recurring_donation_widget.get_comment_edit_form', ['order_item_id' => $order_item_id])->toString();
      $new_content_html = '<div class="add-comment">' .
                      '<a href="' . $add_link_url . '" ' .
                      'data-dialog-type="modal" ' .
                      'data-dialog-options=\'{"width":500}\' ' .
                      'class="use-ajax add-comment-link">' . $this->t('Add Comment') . '</a>' .
                      '</div>';
    }
    else {
      // HTML for comment display with "Edit" link.
      $edit_link_url = Url::fromRoute('commerce_recurring_donation_widget.get_comment_edit_form', ['order_item_id' => $order_item_id])->toString();
      $new_content_html = '<div class="comment-content">' .
                      '<strong>' . $this->t('Comments:') . '</strong> ' . nl2br(htmlspecialchars($comment_value)) .
                      ' <a href="' . $edit_link_url . '" ' .
                      'data-dialog-type="modal" ' .
                      'data-dialog-options=\'{"width":500}\' ' .
                      'class="use-ajax edit-comment-link">' . $this->t('Edit') . '</a>' .
                      '</div>';
    }

    // Replace the inner HTML of the .donation-comment container using HtmlCommand.
    $response->addCommand(new HtmlCommand($selector, $new_content_html));

    return $response;
  }

}
