<?php

namespace Drupal\commerce_recurring_donation_widget\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;

/**
 * Form for editing dedication fields.
 */
class DedicationEditForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new DedicationEditForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dedication_edit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Get values from the form state.
    $order_item_id = $form_state->get('order_item_id');
    $field = $form_state->get('field');
    $current_value = $form_state->get('current_value');
    $dedication_type = $form_state->get('dedication_type');
    $title = $form_state->get('title');

    // Add the form attributes for AJAX.
    $form['#attributes']['class'][] = 'dedication-edit-form';

    // Store params for submit handler.
    $form['order_item_id'] = [
      '#type' => 'hidden',
      '#value' => $order_item_id,
    ];

    $form['field'] = [
      '#type' => 'hidden',
      '#value' => $field,
    ];

    // Build the appropriate fields based on what we're editing.
    if ($field == 'field_dedication_name') {
      $form['value'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Name'),
        '#default_value' => $current_value,
        '#required' => TRUE,
      ];

      // For name edit, also allow changing dedication type.
      $form['dedication_type'] = [
        '#type' => 'radios',
        '#title' => $this->t('Dedication type'),
        '#options' => [
          'honor' => $this->t('In honor of'),
          'memory' => $this->t('In memory of'),
        ],
        '#default_value' => $dedication_type,
        '#required' => TRUE,
      ];
    }
    elseif ($field == 'field_dedication_message') {
      $form['value'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Message'),
        '#default_value' => $current_value,
        '#rows' => 4,
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
      '#attributes' => [
        'class' => ['button--primary'],
      ],
      '#ajax' => [
        'callback' => '::ajaxSubmitCallback',
        'event' => 'click',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Updating...'),
        ],
      ],
    ];

    $form['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#button_type' => 'secondary',
      '#attributes' => [
        'class' => ['dialog-cancel'],
      ],
    ];

    // Add a wrapper for error messages.
    $form['status_messages'] = [
      '#type' => 'status_messages',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $order_item_id = $form_state->getValue('order_item_id');
    $field_being_edited = $form_state->getValue('field');
    $value = trim($form_state->getValue('value'));
    $dedication_type_submitted = $form_state->getValue('dedication_type');

    try {
      /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
      $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($order_item_id);

      if (!$order_item) {
        $this->messenger()->addError($this->t('Order item not found'));
        return;
      }
      $order = $order_item->getOrder();
      if (!$order || $order->getState()->getId() !== 'draft') {
        $this->messenger()->addError($this->t('Order cannot be modified'));
        return;
      }

      // If editing name, and name is cleared, clear also type and message.
      if ($field_being_edited == 'field_dedication_name' && empty($value)) {
        if ($order_item->hasField('field_dedication_name')) {
            $order_item->set('field_dedication_name', NULL);
        }
        if ($order_item->hasField('field_dedication_type')) {
            $order_item->set('field_dedication_type', NULL);
        }
        if ($order_item->hasField('field_dedication_message')) {
            $order_item->set('field_dedication_message', NULL);
        }
      }
      // If editing message, and message is cleared, set it to NULL.
      elseif ($field_being_edited == 'field_dedication_message' && empty($value)) {
        if ($order_item->hasField('field_dedication_message')) {
            $order_item->set('field_dedication_message', NULL);
        }
      }
      // Otherwise, standard update.
      else {
        if ($order_item->hasField($field_being_edited)) {
          $order_item->set($field_being_edited, $value);
        }
        // If name is being edited (and not cleared), also update type.
        if ($field_being_edited == 'field_dedication_name' && $order_item->hasField('field_dedication_type') && $dedication_type_submitted) {
          $order_item->set('field_dedication_type', $dedication_type_submitted);
        }
      }
      $order_item->save();
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('An error occurred: @error', ['@error' => $e->getMessage()]));
      $this->getLogger('commerce_recurring_donation_widget')->error('Error updating dedication: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * AJAX callback for the form submission.
   */
  public function ajaxSubmitCallback(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    if ($form_state->hasAnyErrors()) {
      $response->addCommand(new ReplaceCommand('.dedication-edit-form', $form));
      return $response;
    }

    $order_item_id = $form_state->getValue('order_item_id');
    $response->addCommand(new CloseModalDialogCommand());

    // Reload the order item to get all current values.
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($order_item_id);

    if (!$order_item) {
      // Should not happen if submitForm succeeded, but good to check.
      return $response; 
    }

    $selector = '[data-order-item-id="' . $order_item_id . '"].donation-dedication';
    $new_content_html = '';

    $dedication_name_value = $order_item->hasField('field_dedication_name') ? $order_item->get('field_dedication_name')->value : NULL;
    $dedication_type_value = $order_item->hasField('field_dedication_type') ? $order_item->get('field_dedication_type')->value : NULL;
    $dedication_message_value = $order_item->hasField('field_dedication_message') ? $order_item->get('field_dedication_message')->value : NULL;

    if (empty(trim((string)$dedication_name_value)) || empty(trim((string)$dedication_type_value))) {
      $add_dedication_url = Url::fromRoute('commerce_recurring_donation_widget.get_dedication_edit_form', [
        'order_item_id' => $order_item_id,
        'field' => 'field_dedication_name', // Default to editing name/type when adding
      ])->toString();
      $new_content_html = '<div class="add-dedication">'.
                          '<a href="' . $add_dedication_url . '" class="use-ajax add-dedication-link" data-dialog-type="modal" data-dialog-options=\'{"width":500}\'>' . $this->t('Add Dedication') . '</a>'.
                          '</div>';
    }
    else {
      $dedication_type_label = ($dedication_type_value == 'memory') ? $this->t('In memory of') : $this->t('In honor of');
      $edit_name_url = Url::fromRoute('commerce_recurring_donation_widget.get_dedication_edit_form', [
        'order_item_id' => $order_item_id, 
        'field' => 'field_dedication_name'
      ])->toString();

      $header_html = '<div class="dedication-header">' .
                     '<strong>' . $this->t('Dedication:') . '</strong> ' .
                     $dedication_type_label . ' ' .
                     '<span class="dedication-name">' . htmlspecialchars($dedication_name_value) . '</span>' .
                     ' <a href="' . $edit_name_url . '" class="use-ajax edit-dedication-link" data-dialog-type="modal" data-dialog-options=\'{"width":500}\'>' . $this->t('Edit') . '</a>' .
                     '</div>';

      $message_html = '';
      if ($order_item->hasField('field_dedication_message')) {
        $edit_message_url = Url::fromRoute('commerce_recurring_donation_widget.get_dedication_edit_form', [
            'order_item_id' => $order_item_id, 
            'field' => 'field_dedication_message'
        ])->toString();

        $message_html = '<div class="dedication-message" data-field-name="field_dedication_message">';
        if (!empty(trim((string)$dedication_message_value))) {
          $message_html .= '<strong>' . $this->t('Message:') . '</strong> ' .
                           '<div class="dedication-message-content">' . nl2br(htmlspecialchars($dedication_message_value)) .
                           ' <a href="' . $edit_message_url . '" class="use-ajax edit-dedication-link" data-dialog-type="modal" data-dialog-options=\'{"width":500}\'>' . $this->t('Edit') . '</a>' .
                           '</div>';
        }
        else {
          $message_html .= '<div class="add-dedication-message">'.
                           '<a href="' . $edit_message_url . '" class="use-ajax add-dedication-link" data-dialog-type="modal" data-dialog-options=\'{"width":500}\'>' . $this->t('Add Message') . '</a>'.
                           '</div>';
        }
        $message_html .= '</div>';
      }
      $new_content_html = $header_html . $message_html;
    }

    $response->addCommand(new HtmlCommand($selector, $new_content_html));
    return $response;
  }

}
