<?php

namespace Drupal\commerce_recurring_donation_widget\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;

/**
 * Form for editing designation.
 */
class DesignationEditForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The order item ID.
   *
   * @var int
   */
  protected $orderItemId;

  /**
   * The current designation value.
   *
   * @var string
   */
  protected $currentValue;

  /**
   * The available designation options.
   *
   * @var array
   */
  protected $designationOptions;

  /**
   * Constructs a new DesignationEditForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'designation_edit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $order_item_id = NULL, $current_value = '', array $designation_options = []) {
    $this->orderItemId = $order_item_id;
    $this->currentValue = $current_value;
    $this->designationOptions = $designation_options;

    // Add the form attributes for AJAX.
    $form['#attributes']['class'][] = 'designation-edit-form';

    // Store order item ID for submit handler.
    $form['order_item_id'] = [
      '#type' => 'hidden',
      '#value' => $this->orderItemId,
    ];

    // Designation select list.
    $form['designation'] = [
      '#type' => 'select',
      '#title' => $this->t('Designation'),
      '#options' => $this->designationOptions,
      '#default_value' => $this->currentValue,
      '#required' => FALSE,
      '#empty_option' => $this->t('- None -'),
      '#empty_value' => '',
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
      '#attributes' => [
        'class' => ['button--primary'],
      ],
      '#ajax' => [
        'callback' => '::ajaxSubmitCallback',
        'event' => 'click',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Updating...'),
        ],
      ],
    ];

    $form['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#button_type' => 'secondary',
      '#attributes' => [
        'class' => ['dialog-cancel'],
      ],
    ];

    // Add a wrapper for error messages.
    $form['status_messages'] = [
      '#type' => 'status_messages',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $order_item_id = $form_state->getValue('order_item_id');
    $designation = $form_state->getValue('designation');

    try {
      // Load the order item.
      /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
      $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($order_item_id);

      if (!$order_item) {
        $this->messenger()->addError($this->t('Order item not found'));
        return;
      }

      // Check that the order is still in draft state.
      $order = $order_item->getOrder();
      if (!$order || $order->getState()->getId() !== 'draft') {
        $this->messenger()->addError($this->t('Order cannot be modified'));
        return;
      }

      // Update the designation field.
      if ($order_item->hasField('field_designation')) {
        $order_item->set('field_designation', $designation);
        $order_item->save();
      }
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('An error occurred: @error', ['@error' => $e->getMessage()]));
      $this->getLogger('commerce_recurring_donation_widget')->error('Error updating designation: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * AJAX callback for the form submission.
   */
  public function ajaxSubmitCallback(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    // If there are any errors, re-display the form.
    if ($form_state->hasAnyErrors()) {
      $response->addCommand(new ReplaceCommand('.designation-edit-form', $form));
      return $response;
    }

    $order_item_id = $form_state->getValue('order_item_id');
    $response->addCommand(new CloseModalDialogCommand());

    // Reload the order item to get the current value.
    /** @var \Drupal\commerce_order\Entity\OrderItemInterface $order_item */
    $order_item = $this->entityTypeManager->getStorage('commerce_order_item')->load($order_item_id);

    if (!$order_item) {
      return $response; // Should not happen.
    }

    $selector = '[data-order-item-id="' . $order_item_id . '"].donation-designation';
    $new_content_html = '';
    $designation_value = $order_item->hasField('field_designation') ? $order_item->get('field_designation')->value : NULL;

    if (empty(trim((string)$designation_value))) {
      $add_url = Url::fromRoute('commerce_recurring_donation_widget.get_designation_edit_form', ['order_item_id' => $order_item_id])->toString();
      $new_content_html = '<div class="add-designation">' .
                          '<a href="' . $add_url . '" class="use-ajax add-designation-link" data-dialog-type="modal" data-dialog-options=\'{"width":500}\'>' . $this->t('Add Designation') . '</a>' .
                          '</div>';
    }
    else {
      // Look up the designation label from configuration
      $config = $this->config('commerce_recurring_donation_widget.settings');
      $designations = $config->get('designations') ?: [];
      $designation_label = $designation_value; // Default to the value if label not found

      foreach ($designations as $key => $designation) {
        if (is_numeric($key) && isset($designation['id']) && $designation['id'] === $designation_value) {
          $designation_label = $designation['label'];
          break;
        }
      }

      $edit_url = Url::fromRoute('commerce_recurring_donation_widget.get_designation_edit_form', ['order_item_id' => $order_item_id])->toString();
      $new_content_html = '<div class="donation-designation-content">' .
                          '<strong>' . $this->t('Designation:') . '</strong> ' . htmlspecialchars($designation_label) .
                          ' <a href="' . $edit_url . '" class="use-ajax edit-designation-link" data-dialog-type="modal" data-dialog-options=\'{"width":500}\'>' . $this->t('Edit') . '</a>' .
                          '</div>';
    }

    $response->addCommand(new HtmlCommand($selector, $new_content_html));
    return $response;
  }

}
