<?php

namespace Drupal\commerce_recurring_donation_widget\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Commerce Recurring Donation Widget settings.
 */
class DonationWidgetSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new DonationWidgetSettingsForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   *   The typed config manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $typed_config_manager, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory, $typed_config_manager);
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['commerce_recurring_donation_widget.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'commerce_recurring_donation_widget_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('commerce_recurring_donation_widget.settings');

    // Donation options.
    $form['donation_options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Donation Options'),
      '#tree' => TRUE,
      // Use a wrapper to make it easier to identify and save only the options.
      '#prefix' => '<div id="donation-options-wrapper">',
      '#suffix' => '</div>',
    ];

    $donation_options = $config->get('donation_options') ?: [];
    // Filter out non-numeric keys (like 'actions') if they were somehow saved previously.
    $filtered_donation_options = array_filter($donation_options, 'is_numeric', ARRAY_FILTER_USE_KEY);
    $donation_option_count = $form_state->get('donation_option_count') ?? count($filtered_donation_options);
    $form_state->set('donation_option_count', $donation_option_count);

    // Get the newly added option index, then reset it after checking.
    $newly_added_index = $form_state->get('newly_added_option_index');
    if ($newly_added_index !== NULL) {
      // Reset it after this form build so it's only expanded once.
      $form_state->set('newly_added_option_index', NULL);
    }

    // Get donation type labels from config or use defaults.
    $donation_type_labels = $config->get('donation_type_labels') ?: [
      'one_time' => $this->t('One-time')->render(),
      'monthly' => $this->t('Monthly')->render(),
      'yearly' => $this->t('Yearly')->render(),
      'custom' => $this->t('Custom Amount Options')->render(),
    ];

    // Group options by type.
    $grouped_options = [
      'one_time' => [
        'label' => $donation_type_labels['one_time'] ?? $this->t('One-time'),
        'options' => [],
      ],
      'monthly' => [
        'label' => $donation_type_labels['monthly'] ?? $this->t('Monthly'),
        'options' => [],
      ],
      'yearly' => [
        'label' => $donation_type_labels['yearly'] ?? $this->t('Yearly'),
        'options' => [],
      ],
      'custom' => [
        'label' => $donation_type_labels['custom'] ?? $this->t('Custom Amount Options'),
        'options' => [],
      ],
    ];

    // First, group the options by type.
    for ($i = 0; $i < $donation_option_count; $i++) {
      $type = $filtered_donation_options[$i]['type'] ?? 'one_time';
      $is_custom = $filtered_donation_options[$i]['is_custom_amount'] ?? FALSE;

      if ($is_custom) {
        $grouped_options['custom']['options'][] = $i;
      }
      else {
        $grouped_options[$type]['options'][] = $i;
      }
    }

    // Use the labels from grouped_options for the types.
    $types = [];
    foreach ($grouped_options as $type_key => $type_data) {
      $types[$type_key] = $type_data['label'];
    }

    foreach ($types as $type_key => $type_label) {
      $form['donation_options'][$type_key . '_group'] = [
        '#type' => 'details',
        '#title' => $type_label,
        '#open' => FALSE,
      ];

      if (!empty($grouped_options[$type_key]['options'])) {
        foreach ($grouped_options[$type_key]['options'] as $option_index) {
          $option_title = !empty($filtered_donation_options[$option_index]['title'])
            ? $filtered_donation_options[$option_index]['title']
            : $this->t('Donation Option @num', ['@num' => $option_index + 1]);

          // Add donation type in brackets for custom options.
          if ($type_key === 'custom' && !empty($filtered_donation_options[$option_index]['type'])) {
            $type_label = $types[$filtered_donation_options[$option_index]['type']] ?? '';
            if ($type_label) {
              $option_title .= ' (' . $type_label . ')';
            }
          }

          // Check if this is a newly added item.
          $newly_added_index = $form_state->get('newly_added_option_index');
          $is_newly_added = ($newly_added_index !== NULL && $newly_added_index == $option_index);

          $form['donation_options'][$type_key . '_group'][$option_index] = [
            '#type' => 'details',
            '#title' => $option_title,
            '#open' => $is_newly_added ? TRUE : FALSE,
          ];

          $form['donation_options'][$type_key . '_group'][$option_index]['title'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Title'),
            '#default_value' => $filtered_donation_options[$option_index]['title'] ?? '',
            '#required' => TRUE,
          ];

          $form['donation_options'][$type_key . '_group'][$option_index]['variation_id'] = [
            '#type' => 'entity_autocomplete',
            '#title' => $this->t('Product Variation'),
            '#target_type' => 'commerce_product_variation',
            '#default_value' => isset($filtered_donation_options[$option_index]['variation_id']) && $filtered_donation_options[$option_index]['variation_id'] ?
            $this->entityTypeManager->getStorage('commerce_product_variation')->load($filtered_donation_options[$option_index]['variation_id']) : NULL,
            '#required' => TRUE,
          ];

          $form['donation_options'][$type_key . '_group'][$option_index]['type'] = [
            '#type' => 'radios',
            '#title' => $this->t('Donation Type'),
            '#options' => [
              'one_time' => $this->t('One-time'),
              'monthly' => $this->t('Monthly'),
              'yearly' => $this->t('Yearly'),
            ],
            '#default_value' => $filtered_donation_options[$option_index]['type'] ?? 'one_time',
            '#required' => TRUE,
          ];

          // Add remove button for this specific option.
          $form['donation_options'][$type_key . '_group'][$option_index]['remove_button'] = [
            '#type' => 'submit',
            '#value' => $this->t('Remove item'),
            '#name' => 'remove_option_' . $option_index,
            '#submit' => ['::removeSpecificOption'],
            '#option_index' => $option_index,
            '#ajax' => [
              'callback' => '::ajaxRefreshOptions',
              'wrapper' => 'donation-options-wrapper',
            ],
            '#limit_validation_errors' => [],
            '#attributes' => ['class' => ['button--small']],
          ];
        }
      }
      else {
        $form['donation_options'][$type_key . '_group']['no_items'] = [
          '#markup' => '<p>' . $this->t('No items yet.') . '</p>',
        ];
      }

      // Add "Add" button for this category.
      $form['donation_options'][$type_key . '_group']['add_button'] = [
        '#type' => 'submit',
        '#value' => $type_key === 'custom'
          ? $this->t('Add to Custom Donations')
          : $this->t('Add to @type', ['@type' => $type_label]),
        '#name' => 'add_option_to_' . $type_key,
        '#submit' => ['::addOptionToCategory'],
        '#category' => $type_key,
        '#ajax' => [
          'callback' => '::ajaxRefreshOptions',
          'wrapper' => 'donation-options-wrapper',
        ],
        '#limit_validation_errors' => [],
      ];
    }

    // Global Display Settings - these apply site-wide to all donation widgets
    $form['display_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Global Display Settings'),
      '#description' => $this->t('These settings apply to all donation widgets across the site.'),
      '#tree' => TRUE,
    ];

    $display_settings = $config->get('display_settings') ?: [];

    $form['display_settings']['enable_dedication'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable dedication option'),
      '#description' => $this->t('Allow donors to dedicate their donation "in honor of" or "in memory of" someone.'),
      '#default_value' => !empty($display_settings['enable_dedication']),
    ];

    $form['display_settings']['enable_comments'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable comments option'),
      '#description' => $this->t('Allow donors to add a comment to their donation.'),
      '#default_value' => !empty($display_settings['enable_comments']),
    ];

    $form['display_settings']['enable_designation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable designation option'),
      '#description' => $this->t('Allow donors to select a designation (fund/purpose) for their donation.'),
      '#default_value' => !empty($display_settings['enable_designation']),
    ];

    $form['display_settings']['enable_custom_amount'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable custom amount entry'),
      '#description' => $this->t('Allow donors to enter their own donation amount.'),
      '#default_value' => !empty($display_settings['enable_custom_amount']),
    ];

    $form['display_settings']['custom_amount_placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom amount placeholder'),
      '#description' => $this->t('Placeholder text for the custom amount input field.'),
      '#default_value' => $display_settings['custom_amount_placeholder'] ?? $this->t('Enter amount'),
      '#states' => [
        'visible' => [
          ':input[name="display_settings[enable_custom_amount]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Donation Type Labels.
    $form['donation_type_labels'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Donation Type Labels'),
      '#description' => $this->t('Customize the labels used for each donation type.'),
      '#tree' => TRUE,
    ];

    $donation_type_labels = $config->get('donation_type_labels') ?: [];

    $form['donation_type_labels']['one_time'] = [
      '#type' => 'textfield',
      '#title' => $this->t('One-time donation label'),
      '#default_value' => $donation_type_labels['one_time'] ?? $this->t('One-time'),
      '#required' => TRUE,
    ];

    $form['donation_type_labels']['monthly'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Monthly donation label'),
      '#default_value' => $donation_type_labels['monthly'] ?? $this->t('Monthly'),
      '#required' => TRUE,
    ];

    $form['donation_type_labels']['yearly'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Yearly donation label'),
      '#default_value' => $donation_type_labels['yearly'] ?? $this->t('Yearly'),
      '#required' => TRUE,
    ];

    $form['donation_type_labels']['custom'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom amount label'),
      '#default_value' => $donation_type_labels['custom'] ?? $this->t('Custom Amount Options'),
      '#required' => TRUE,
    ];

    // Designations.
    $form['designations'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Designations'),
      '#tree' => TRUE,
      '#prefix' => '<div id="designations-wrapper">',
      '#suffix' => '</div>',
    ];

    $designations = $config->get('designations') ?: [];
    $filtered_designations = array_filter($designations, 'is_numeric', ARRAY_FILTER_USE_KEY);
    $designation_count = $form_state->get('designation_count') ?? count($filtered_designations);
    $form_state->set('designation_count', $designation_count);

    for ($i = 0; $i < $designation_count; $i++) {
      $form['designations'][$i] = [
        '#type' => 'container',
        '#attributes' => ['class' => ['container-inline']],
      ];

      $form['designations'][$i]['label'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Label'),
        '#default_value' => $filtered_designations[$i]['label'] ?? '',
        '#size' => 40,
      ];

      $form['designations'][$i]['id'] = [
        '#type' => 'machine_name',
        '#title' => $this->t('ID'),
        '#default_value' => $filtered_designations[$i]['id'] ?? 'designation_' . ($i + 1),
        '#machine_name' => [
          'exists' => [$this, 'designationExists'],
          'source' => ['designations', $i, 'label'],
        ],
        '#size' => 20,
      ];
    }

    $form['designations']['actions'] = [
      '#type' => 'actions',
    ];

    $form['designations']['actions']['add_designation'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another designation'),
      '#submit' => ['::addDesignation'],
      '#ajax' => [
        'callback' => '::ajaxRefreshDesignations',
        'wrapper' => 'designations-wrapper',
      ],
      '#limit_validation_errors' => [],
    ];

    if ($designation_count > 0) {
          $form['designations']['actions']['remove_designation'] = [
      '#type' => 'submit',
      '#value' => $this->t('Remove last designation'),
      '#submit' => ['::removeDesignation'],
      '#ajax' => [
        'callback' => '::ajaxRefreshDesignations',
        'wrapper' => 'designations-wrapper',
      ],
      '#limit_validation_errors' => [],
    ];
  }

  $form['designations']['default_designation'] = [
    '#type' => 'select',
    '#title' => $this->t('Default designation'),
    '#options' => $this->getDesignationOptions($form, $form_state),
    '#default_value' => $config->get('default_designation'),
    '#empty_option' => $this->t('- None -'),
  ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback for refreshing donation options.
   */
  public function ajaxRefreshOptions(array &$form, FormStateInterface $form_state) {
    return $form['donation_options'];
  }

  /**
   * AJAX callback for refreshing designations.
   */
  public function ajaxRefreshDesignations(array &$form, FormStateInterface $form_state) {
    return $form['designations'];
  }

  /**
   * Gets the designation options.
   */
  protected function getDesignationOptions(array $form, FormStateInterface $form_state) {
    $options = [];
    $values = $form_state->getValues();
    $designations_from_form = $values['designations'] ?? NULL;

    if ($designations_from_form) {
      foreach ($designations_from_form as $key => $designation) {
        // Only process actual designation entries, not 'actions' etc.
        if (is_numeric($key) && !empty($designation['id']) && !empty($designation['label'])) {
          $options[$designation['id']] = $designation['label'];
        }
      }
    }
    else {
      // Fallback to config if form values are not yet available (e.g. initial load)
      $config = $this->config('commerce_recurring_donation_widget.settings');
      $designations_from_config = $config->get('designations') ?: [];
      foreach ($designations_from_config as $key => $designation) {
        if (is_numeric($key) && !empty($designation['id']) && !empty($designation['label'])) {
          $options[$designation['id']] = $designation['label'];
        }
      }
    }
    return $options;
  }

  /**
   * Callback for checking if a designation already exists.
   */
  public function designationExists($value, array $element, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if (!isset($values['designations'])) {
      return FALSE;
    }

    $count = 0;
    foreach ($values['designations'] as $key => $designation) {
      if (is_numeric($key) && isset($designation['id']) && $designation['id'] === $value) {
        $count++;
      }
    }
    return $count > 1;
  }

  /**
   * Submit handler for adding a donation option to a specific category.
   */
  public function addOptionToCategory(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $category = $triggering_element['#category'];
    $count = $form_state->get('donation_option_count');

    // Get current options from config.
    $config = $this->config('commerce_recurring_donation_widget.settings');
    $donation_options = $config->get('donation_options') ?: [];

    // Add a new empty option at the end with the specified category.
    $is_custom = ($category === 'custom');
    $donation_options[] = [
      'title' => '',
      'variation_id' => '',
    // For custom options, set a default type.
      'type' => $is_custom ? 'one_time' : $category,
      'is_custom_amount' => $is_custom,
    ];

    // Save the updated options.
    $config = $this->configFactory->getEditable('commerce_recurring_donation_widget.settings');
    $config->set('donation_options', $donation_options);
    $config->save();

    // Update the count.
    $form_state->set('donation_option_count', $count + 1);

    // Store the index of the newly added option so it can be expanded.
    $form_state->set('newly_added_option_index', count($donation_options) - 1);

    $form_state->setRebuild(TRUE);
  }

  /**
   * Submit handler for removing a specific donation option.
   */
  public function removeSpecificOption(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $option_index = $triggering_element['#option_index'];

    // Get current options from config.
    $config = $this->config('commerce_recurring_donation_widget.settings');
    $donation_options = $config->get('donation_options') ?: [];

    // Remove the option at the specified index.
    if (isset($donation_options[$option_index])) {
      unset($donation_options[$option_index]);
      // Re-index the array to ensure sequential keys.
      $donation_options = array_values($donation_options);

      // Save the updated options.
      $config = $this->configFactory->getEditable('commerce_recurring_donation_widget.settings');
      $config->set('donation_options', $donation_options);
      $config->save();

      // Update the count.
      $count = $form_state->get('donation_option_count');
      $form_state->set('donation_option_count', $count - 1);
    }

    $form_state->setRebuild(TRUE);
  }

  /**
   * Submit handler for adding a designation.
   */
  public function addDesignation(array &$form, FormStateInterface $form_state) {
    $count = $form_state->get('designation_count');
    $form_state->set('designation_count', $count + 1);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Submit handler for removing a designation.
   */
  public function removeDesignation(array &$form, FormStateInterface $form_state) {
    $count = $form_state->get('designation_count');
    if ($count > 0) {
      $form_state->set('designation_count', $count - 1);
    }
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('commerce_recurring_donation_widget.settings');

    // Save donation options.
    $donation_options_values = $form_state->getValue('donation_options');
    $donation_options_to_save = [];
    $indexed_options = [];

    if (isset($donation_options_values) && is_array($donation_options_values)) {
      // Define the group keys we need to look for.
      $group_keys = ['one_time_group', 'monthly_group', 'yearly_group', 'custom_group'];

      // Extract options from each group and preserve their original indices.
      foreach ($group_keys as $group_key) {
        if (isset($donation_options_values[$group_key]) && is_array($donation_options_values[$group_key])) {
          foreach ($donation_options_values[$group_key] as $option_index => $option_values) {
            if (is_numeric($option_index) && is_array($option_values)) {

              // Get the is_custom_amount value from existing configuration.
              $existing_options = $config->get('donation_options') ?: [];
              $is_custom_amount = FALSE;

              foreach ($existing_options as $existing_option) {
                if (isset($existing_option['variation_id']) && $existing_option['variation_id'] == $option_values['variation_id']) {
                  $is_custom_amount = !empty($existing_option['is_custom_amount']);
                  break;
                }
              }

              $indexed_options[$option_index] = [
                'title' => $option_values['title'],
                'variation_id' => $option_values['variation_id'],
                'type' => $option_values['type'],
              // Preserve the existing value.
                'is_custom_amount' => $is_custom_amount,
              ];
            }
          }
        }
      }

      // Sort options by their original indices to maintain order.
      ksort($indexed_options);

      // Convert to sequential array for saving.
      $donation_options_to_save = array_values($indexed_options);
    }

    $config->set('donation_options', $donation_options_to_save);

    // Save display settings (now global settings).
    $display_settings_values = $form_state->getValue('display_settings');
    $config->set('display_settings', $display_settings_values);

    // Save designations.
    $designations_values = $form_state->getValue('designations');
    $cleaned_designations = [];
    if (is_array($designations_values)) {
      foreach ($designations_values as $key => $value) {
        if (is_numeric($key) && is_array($value) && !empty($value['id']) && !empty($value['label'])) {
          $cleaned_designations[] = $value;
        }
      }
    }
    $config->set('designations', $cleaned_designations);

    $config->set('default_designation', $form_state->getValue('default_designation'));

    // Save donation type labels.
    $donation_type_labels_values = $form_state->getValue('donation_type_labels');
    $config->set('donation_type_labels', $donation_type_labels_values);

    $config->save();

    parent::submitForm($form, $form_state);
  }

}
