<?php

namespace Drupal\commerce_recurring_donation_widget\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Url;

/**
 * Provides a 'DonationWidgetBlock' block.
 *
 * @Block(
 *   id = "commerce_recurring_donation_widget_block",
 *   admin_label = @Translation("Recurring Donation Widget"),
 *   category = @Translation("Commerce")
 * )
 */
class DonationWidgetBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a new DonationWidgetBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'block_settings' => [
        'title' => 'Donate',
        'donation_options' => [],
        'custom_amount_label' => $this->t('Custom Amount'),
        'designations' => [],
        'default_designation' => '',
      ],
    ];
  }

  /**
   * Checks if the module has been configured.
   *
   * @return bool
   *   TRUE if the module has been configured, FALSE otherwise.
   */
  protected function isConfigured() {
    $settings = $this->configFactory->get('commerce_recurring_donation_widget.settings');
    if ($settings->isNew()) {
      return FALSE;
    }
    $donation_options = $settings->get('donation_options') ?: [];

    // Ensure donation_options is an array and not empty after filtering non-data.
    $filtered_options = array_filter($donation_options, function ($option) {
        return is_array($option) && isset($option['title']);
    });
    return !empty($filtered_options);
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    if (!$this->isConfigured()) {
      $form['no_settings'] = [
        '#type' => 'markup',
        '#markup' => $this->t('<div class="messages messages--warning">You need to <a href="@url">configure the donation widget settings</a> (and add at least one option) before using this block.</div>',
          ['@url' => Url::fromRoute('commerce_recurring_donation_widget.settings')->toString()]),
      ];
      return $form;
    }

    $settings = $this->configFactory->get('commerce_recurring_donation_widget.settings');

    $form['block_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Block Settings'),
      '#tree' => TRUE,
    ];

    // Note: Custom amount placeholder is now controlled globally via the main module configuration

    // Donation options with nested structure.
    $form['block_settings']['donation_options_wrapper'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Donation Options'),
      '#tree' => TRUE,
      '#prefix' => '<div id="donation-options-wrapper">',
      '#suffix' => '</div>',
    ];

    $donation_options_from_settings = $settings->get('donation_options') ?: [];

    // Group options by type.
    $donation_type_labels = $settings->get('donation_type_labels') ?: [
      'one_time' => $this->t('One-time')->render(),
      'monthly' => $this->t('Monthly')->render(),
      'yearly' => $this->t('Yearly')->render(),
      'custom' => $this->t('Custom Amount Options')->render(),
    ];

    // Group options by type.
    $grouped_options = [
      'one_time' => [
        'label' => $donation_type_labels['one_time'] ?? $this->t('One-time'),
        'options' => [],
      ],
      'monthly' => [
        'label' => $donation_type_labels['monthly'] ?? $this->t('Monthly'),
        'options' => [],
      ],
      'yearly' => [
        'label' => $donation_type_labels['yearly'] ?? $this->t('Yearly'),
        'options' => [],
      ],
      'custom' => [
        'label' => $donation_type_labels['custom'] ?? $this->t('Custom Amount Options'),
        'options' => [],
      ],
    ];

    // First, group the options by type.
    foreach ($donation_options_from_settings as $key => $option) {
      if (is_array($option) && isset($option['title']) && isset($option['type'])) {
        $type = $option['type'] ?? 'one_time';
        $is_custom = $option['is_custom_amount'] ?? FALSE;

        if ($is_custom) {
          $grouped_options['custom']['options'][$key] = $option;
        }
        else {
          $grouped_options[$type]['options'][$key] = $option;
        }
      }
    }

    // Prepare default values for the form checkboxes.
    $saved_block_options = $config['block_settings']['donation_options'] ?? [];

    // Create a details element for each donation type group.
    foreach ($grouped_options as $type_key => $type_data) {
      $form['block_settings']['donation_options_wrapper'][$type_key . '_group'] = [
        '#type' => 'details',
        '#tree' => TRUE,
        '#title' => $type_data['label'],
        '#open' => FALSE,
      ];

      if (!empty($type_data['options'])) {
        $options_for_checkboxes = [];
        $default_values_for_form = [];

        foreach ($type_data['options'] as $option_key => $option) {
          $options_for_checkboxes[$option_key] = $option['title'] . ' (' . $this->t('@type', ['@type' => $option['type']]) . ')';

          // Default to unselected.
          $default_values_for_form[$option_key] = 0;
          // If this option is in the saved block options, mark it as selected.
          if (isset($saved_block_options[$option_key])) {
            $default_values_for_form[$option_key] = 1;
          }
        }

        // Build checkboxes manually for better control.
        $form['block_settings']['donation_options_wrapper'][$type_key . '_group']['options'] = [
          '#type' => 'container',
          '#tree' => TRUE,
          '#title' => $this->t('Select options to display'),
          '#attributes' => ['class' => ['donation-options-wrapper']],
        ];

        foreach ($options_for_checkboxes as $key => $label) {
          $form['block_settings']['donation_options_wrapper'][$type_key . '_group']['options'][$key] = [
            '#type' => 'checkbox',
            '#title' => $label,
            '#return_value' => 1,
            '#default_value' => in_array($key, $saved_block_options) ? 1 : 0,
          ];
        }
      }
      else {
        $form['block_settings']['donation_options_wrapper'][$type_key . '_group']['no_items'] = [
          '#markup' => '<p>' . $this->t('No items in this category.') . '</p>',
        ];
      }
    }

    // Note: Display settings (dedication, comments, designation) are now controlled globally
    // via the main module configuration at /admin/commerce/config/donation-widget

    // Designations.
    $form['block_settings']['designations_wrapper'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Designations'),
      '#tree' => TRUE,
      '#prefix' => '<div id="designations-wrapper">',
      '#suffix' => '</div>',
      // Designations are now always available if the global setting is enabled
      // The visibility is controlled by the global configuration
    ];

    $designations_from_settings = $settings->get('designations') ?: [];
    $designation_options_for_checkboxes = [];
    foreach ($designations_from_settings as $designation) {
      if (is_array($designation) && !empty($designation['id']) && !empty($designation['label'])) {
        $designation_options_for_checkboxes[$designation['id']] = $designation['label'];
      }
    }

    $form['block_settings']['designations_wrapper']['designations'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Designations to Display'),
      '#options' => $designation_options_for_checkboxes,
      '#default_value' => $config['block_settings']['designations'] ?? array_keys($designation_options_for_checkboxes),
    ];

    $form['block_settings']['designations_wrapper']['default_designation'] = [
      '#type' => 'select',
      '#title' => $this->t('Default Designation'),
      '#options' => $designation_options_for_checkboxes,
      '#empty_option' => $this->t('- None -'),
      '#default_value' => $config['block_settings']['default_designation'] ?? ($settings->get('default_designation') ?? NULL),
    ];

    return $form;
  }

  /**
   * Validates the donation options form element.
   *
   * @param array $element
   *   The form element being processed.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $complete_form
   *   The complete form structure.
   */
  public function validateDonationOptions(array &$element, FormStateInterface $form_state, array &$complete_form) {
    $value = $form_state->getValue($element['#parents']);

    if (is_array($value)) {
      // Get the allowed options from the element.
      $allowed_options = array_keys($element['#options']);

      // Filter out any values that aren't in the allowed options.
      $filtered_value = array_filter($value, function ($item) use ($allowed_options) {
        return $item !== 0 && $item !== '0' && $item !== '' &&
               $item !== NULL && in_array($item, $allowed_options, TRUE);
      });

      // Reset array keys to ensure proper JSON encoding.
      $filtered_value = array_values($filtered_value);

      // Set the filtered value back to the form state.
      $form_state->setValueForElement($element, $filtered_value);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    if ($form_state->hasValue('block_settings')) {
      $block_settings = $form_state->getValue('block_settings');

      // Process donation options from the nested structure.
      $donation_options = [];
      // Extract the submitted values from the block settings.
      $donation_wrapper = $block_settings['donation_options_wrapper'] ?? [];
      if (is_array($donation_wrapper)) {
        foreach ($donation_wrapper as $group_key => $group_data) {
          if (!empty($group_data['options']) && is_array($group_data['options'])) {
            foreach ($group_data['options'] as $option_key => $value) {
              // Only include selected options (value is the option_key when checked)
              if ($value && $value !== 0 && $value !== '0') {
                $donation_options[$option_key] = $option_key;
              }
            }
          }
        }
      }

      // Set the processed donation options.
      $block_settings['donation_options'] = $donation_options;

      // Note: All display settings are now controlled globally via the main module configuration

      // Process designations.
      $designations = $form_state->getValue(['block_settings', 'designations_wrapper', 'designations']);
      if (!empty($designations)) {
        // For checkboxes, filter out the 0 values (unselected options)
        $block_settings['designations'] = array_filter($designations, function ($value) {
          return $value !== 0;
        });
      }

      // Process default designation.
      $default_designation = $form_state->getValue(['block_settings', 'designations_wrapper', 'default_designation']);
      // Always store the value, even if empty string.
      $block_settings['default_designation'] = (string) $default_designation;

      // Remove the wrapper elements from the configuration.
      unset($block_settings['donation_options_wrapper']);
      unset($block_settings['designations_wrapper']);

      $this->configuration['block_settings'] = $block_settings;
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowed();
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    if (!$this->isConfigured()) {
      return [
        '#markup' => $this->t('Please <a href="@url">configure the donation widget settings</a> (and add at least one option) first.',
          ['@url' => Url::fromRoute('commerce_recurring_donation_widget.settings')->toString()]),
      ];
    }

    $config = $this->getConfiguration();
    $settings = $this->configFactory->get('commerce_recurring_donation_widget.settings');
    $display_settings = $settings->get('display_settings') ?: [];

    // Determine enabled option keys from block settings.
    $enabled_options_map = $config['block_settings']['donation_options'] ?? [];
    // The values in the array are the actual option IDs we want to enable.
    $enabled_option_ids = array_values($enabled_options_map);

    $global_donation_options = $settings->get('donation_options') ?: [];
    $donation_options_for_template = [];
    
    // Check if custom amounts are globally enabled
    $custom_amount_enabled = $display_settings['enable_custom_amount'] ?? FALSE;

    foreach ($global_donation_options as $key => $option_data) {
      // Critical fix: Ensure $option_data is an array and has the required keys.
      if (is_array($option_data) &&
          isset($option_data['title']) &&
          isset($option_data['variation_id']) &&
          isset($option_data['type']) &&
          (in_array($key, $enabled_option_ids, TRUE) || in_array((string) $key, $enabled_option_ids, TRUE))) {

        // Skip custom amount options if custom amounts are globally disabled
        $is_custom_option = !empty($option_data['is_custom_amount']);
        if ($is_custom_option && !$custom_amount_enabled) {
          continue;
        }

        $donation_options_for_template[] = [
          'amount' => $option_data['title'],
          'variation_id' => $option_data['variation_id'],
          'type' => $option_data['type'],
          'is_custom_amount' => $is_custom_option,
          'title' => $option_data['title'],
        ];
      }
      elseif (!is_array($option_data) && (in_array($key, $enabled_option_ids, TRUE) || in_array((string) $key, $enabled_option_ids, TRUE))) {
        // Log if an enabled option is not an array (e.g. might be 'actions' string).
        $this->loggerFactory->get('commerce_recurring_donation_widget')->warning('Enabled donation option with key @key is not structured as an array. Value: @value', [
          '@key' => $key,
          '@value' => 'processed',
        ]);
      }
    }

    if (empty($donation_options_for_template)) {
      if (!empty($global_donation_options) && empty($enabled_option_keys)) {
        return [
          '#markup' => $this->t('No donation options are selected for this block. Please edit the block configuration to select donation options.'),
        ];
      }
      return [
        '#markup' => $this->t('No donation options are available. Please configure the donation widget settings and then edit this block to select options.'),
      ];
    }

    // Determine unique donation types available from the filtered options
    // Only include donation types that have at least one enabled option.
    $present_donation_types = [];
    $donation_type_labels = $settings->get('donation_type_labels') ?: [];

    // Group enabled donation options by type.
    $enabled_options_by_type = [];
    foreach ($donation_options_for_template as $option) {
      if (!empty($option['type']) && is_string($option['type'])) {
        $enabled_options_by_type[$option['type']][] = $option;
      }
    }

    // Only include types that have at least one enabled option.
    foreach ($enabled_options_by_type as $type => $options) {
      if (!empty($options)) {
        // Use the label from configuration if available, otherwise fallback to generated label.
        $present_donation_types[$type] = $donation_type_labels[$type] ?? $this->t(ucfirst($type));
      }
    }

    // Order them consistently: one-time, monthly, yearly for display.
    $ordered_present_types = [];
    $type_order = ['one_time', 'monthly', 'yearly'];
    foreach ($type_order as $type_key) {
      if (isset($present_donation_types[$type_key])) {
        $ordered_present_types[$type_key] = $present_donation_types[$type_key];
      }
    }

    if (empty($ordered_present_types)) {
      // This case should ideally be caught by the empty($donation_options_for_template) check earlier,
      // but as a fallback if somehow options exist but types are malformed.
      return [
        '#markup' => $this->t('No valid donation types found for the configured options.'),
      ];
    }

    $active_donation_type = array_key_first($ordered_present_types);

    // Get enabled designations directly from their keys.
    $enabled_designation_ids = array_keys($config['block_settings']['designations'] ?? []);
    // Ensure all keys are strings for consistent comparison.
    $enabled_designation_ids = array_map('strval', $enabled_designation_ids);

    $global_designations = $settings->get('designations') ?: [];
    $designation_options_for_template = [];

    // If enable_designation is true, include all available designations if none are specifically selected.
    $show_all_designations = !empty($config['block_settings']['enable_designation']) && empty($enabled_designation_ids);

    foreach ($global_designations as $designation) {
      if (is_array($designation) && !empty($designation['id'])) {
        // Cast designation ID to string for consistent comparison.
        $designation_id = (string) $designation['id'];
        if ($show_all_designations || in_array($designation_id, $enabled_designation_ids, TRUE)) {
          $designation_options_for_template[$designation_id] = $designation['label'];
        }
      }
    }

    // Get default designation.
    $default_designation = '';
    if (isset($config['block_settings']['default_designation'])) {
      $designation_value = (string) $config['block_settings']['default_designation'];
      // Check if the designation exists in available options.
      if ($designation_value !== '' && isset($designation_options_for_template[$designation_value])) {
        $default_designation = $designation_value;
      }
    }

    // Use global settings for display features
    $enable_dedication = !empty($display_settings['enable_dedication']);
    $enable_comments = !empty($display_settings['enable_comments']);
    $enable_designation = !empty($display_settings['enable_designation']);

    // Filter donation options to only include those of the active donation type.
    $filtered_donation_options = array_filter($donation_options_for_template, function ($option) use ($active_donation_type) {
      return $option['type'] === $active_donation_type;
    });

    // Create a map of donation options by type for JavaScript.
    $donation_options_by_type = [];
    foreach ($donation_options_for_template as $option) {
      if (!empty($option['type']) && is_string($option['type'])) {
        $donation_options_by_type[$option['type']][] = $option;
      }
    }

    $build = [
      '#theme' => 'commerce_recurring_donation_widget',
      '#donation_options' => $filtered_donation_options,
      '#donation_types' => $ordered_present_types,
      '#active_donation_type' => $active_donation_type,
      '#custom_amount_globally_enabled' => $custom_amount_enabled,
      '#custom_amount_placeholder' => $display_settings['custom_amount_placeholder'] ?? $this->t('Enter amount'),
      '#custom_amount_label' => $config['block_settings']['custom_amount_label'] ?? $this->t('Custom Amount'),
      '#enable_dedication' => $enable_dedication,
      '#enable_comments' => $enable_comments,
      '#enable_designation' => $enable_designation,
      '#designation_options' => $designation_options_for_template,
      '#default_designation' => $default_designation,
      '#all_donation_options' => $donation_options_for_template,

      '#attached' => [
        'library' => ['commerce_recurring_donation_widget/donation_widget'],
        'drupalSettings' => [
          'commerceRecurringDonationWidget' => [
            'addToCartUrl' => Url::fromRoute('commerce_recurring_donation_widget.add_to_cart')->toString(),
            'customAmountGloballyEnabled' => $custom_amount_enabled,
            'donationOptionsByType' => $donation_options_by_type,
            'activeDonationType' => $active_donation_type,
          ],
        ],
      ],
      // Add cache metadata to ensure the block rebuilds when configuration changes.
      '#cache' => [
        'tags' => ['config:block.block.' . $this->getPluginId()],
        'contexts' => ['url'],
      ],
    ];

    return $build;
  }

}
